import React from 'react';
import {
  Button,
  Col,
  Form,
  Collapse,
  Input,
  Row,
  Select,
  Tabs,
  Tag,
  Typography,
  Drawer,
  FormInstance,
} from 'antd';
import styles from '@/pages/Product/MyProducts/styles.module.less';
import { useDispatch, useIntl, useSelector } from 'umi';
import {
  IMyProductState,
  IMyProduct,
  EditProductVariant,
  IEditProductData,
} from '@/typings';
import { myProductSave } from '@/pages/Product/MyProducts/models';
import { useSafeState } from 'ahooks';
import { history } from '@@/core/history';
import ImageUpload from '@/components/ImageUpload';
import { productModelToStateDataTransformer } from '@/utils/ProductModelDataTransformer';

const { Panel } = Collapse;

const dataToVariantFormValues = (variants: EditProductVariant[] | null) => {
  const initValues: { [name: string]: any } = {};
  if (!variants) return {};
  variants.forEach((e, index) => {
    initValues[`${index}_variantAttribute`] = e.variantAttribute;
    initValues[`${index}_variantName`] = e.variantName;
    initValues[`${index}_simpleSku`] = e.simpleSku;
    initValues[`${index}_weight`] = e.weight;
    initValues[`${index}_height`] = e.height;
    initValues[`${index}_width`] = e.width;
    initValues[`${index}_length`] = e.length;
  });
  return initValues;
};

const dataToPricingFormValues = (variants: EditProductVariant[] | null) => {
  const initValues: { [name: string]: any } = {};
  if (!variants) return {};
  variants.forEach((e, index) => {
    initValues[`${index}_unitPrice`] = e.unitPrice;
    e.tierPricing!.map((tier, tierIndex) => {
      Object.entries(tier).forEach(([attributeName, value]) => {
        let copy: any = value;
        if (Array.isArray(value)) copy = value.map((x) => x.toString()).join('-');
        initValues[`${index}_${tierIndex}_${attributeName}`] = copy;
      });
    });
  });
  return initValues;
};

// eslint-disable-next-line @typescript-eslint/no-unused-vars
const AddOnSelect = ({
  disable /*ref, packageObj*/,
}: {
  disable: boolean;
  ref: React.MutableRefObject<FormInstance<any>>;
  packageObj: any;
}) => {
  return (
    <Select disabled={disable} defaultValue={'cm'}>
      <Select.Option key={'cm'}>cm</Select.Option>
    </Select>
  );
};

const getInitialValue = (editItemData?: IEditProductData) => {
  if (!editItemData) return {};
  const generalDetails = editItemData.generalDetails;
  const labelDetails = editItemData.labelDetails;
  const variants: any = dataToVariantFormValues(editItemData.variants);
  const pricing: any = dataToPricingFormValues(editItemData.variants);
  return {
    generalDetails,
    labelDetails,
    variants,
    pricing,
  };
};

const EditDrawer = () => {
  const dispatch = useDispatch();
  const intl = useIntl();

  const generalDetailsRef = React.useRef() as React.MutableRefObject<FormInstance<any>>;
  const labelDetailsRef = React.useRef() as React.MutableRefObject<FormInstance<any>>;
  const variantsRef = React.useRef() as React.MutableRefObject<FormInstance<any>>;
  const pricingRef = React.useRef() as React.MutableRefObject<FormInstance<any>>;

  const myProduct = useSelector<IMyProductState, IMyProduct>((state) => {
    return state.myProduct;
  });

  const [activeDrawerKey, setActiveDrawerKey] = useSafeState('1');
  const [disable] = useSafeState<boolean>(true);

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  const onUpload = (fileKey: string, fileName: string, size: number) => {};

  const editProductData = myProduct.openDrawerItem
    ? productModelToStateDataTransformer(myProduct.openDrawerItem)
    : undefined;

  const { generalDetails, labelDetails, variants, pricing } =
    getInitialValue(editProductData);

  return (
    <Drawer
      destroyOnClose={true}
      title={
        <Row>
          <Col span={24} className={styles.tinyDrawerTitle}>
            {intl.formatMessage({ id: 'product.myProduct' })}
          </Col>
          <Col span={24} className={styles.drawerTitleContainer}>
            <Typography.Title level={4}>{`${myProduct.openDrawerItem?.productName} by ${
              myProduct.openDrawerItem?.factoryName || '-'
            }`}</Typography.Title>
            <Tag>{intl.formatMessage({ id: 'product.draft' })}</Tag>
          </Col>
        </Row>
      }
      placement={'right'}
      closable={true}
      onClose={() =>
        dispatch({ type: myProductSave, payload: { openDrawerItem: undefined } })
      }
      visible={myProduct.openDrawerItem !== undefined}
      maskClosable={true}
      size={'large'}
      extra={
        <Button
          onClick={() => {
            // if (disable) return setDisable(false);
            // message.success(intl.formatMessage({ id: 'product.success' }));
            // setDisable(true);
            dispatch({ type: myProductSave, payload: { openDrawerItem: undefined } });
            history.push(
              `/home/product/edit-product/${myProduct.openDrawerItem?.identifier}`,
            );
          }}
        >
          {intl.formatMessage({ id: disable ? 'product.edit' : 'product.save' })}
        </Button>
      }
      className={styles.drawer}
    >
      <Tabs activeKey={activeDrawerKey} onChange={(e) => setActiveDrawerKey(e)}>
        {/* general details */}
        <Tabs.TabPane tab={intl.formatMessage({ id: 'product.generalDetails' })} key="1">
          <Form
            initialValues={{ ...generalDetails }}
            ref={generalDetailsRef}
            labelAlign={'left'}
            wrapperCol={{ span: 16 }}
            labelCol={{ span: 8 }}
          >
            <Form.Item
              name={'configurableSku'}
              label={intl.formatMessage({ id: 'product.configurableSku' })}
              rules={[{ required: true }]}
            >
              <Input disabled={disable} />
            </Form.Item>
            <Form.Item
              name={'factory'}
              label={intl.formatMessage({ id: 'product.factory' })}
              rules={[{ required: false }]}
            >
              <Input disabled={disable} />
            </Form.Item>
            <Form.Item
              name={'factory'}
              label={intl.formatMessage({ id: 'product.description' })}
              rules={[{ required: false }]}
            >
              <Input.TextArea disabled={disable} />
            </Form.Item>
            <Form.Item
              name={'countryOfOrigin'}
              label={intl.formatMessage({ id: 'product.countryOfOrigin' })}
              rules={[{ required: true }]}
            >
              <Select disabled={disable}>
                <Select.Option key={'china'}>China</Select.Option>
                <Select.Option key={'usa'}>USA</Select.Option>
              </Select>
            </Form.Item>
            <Form.Item
              name={'otherNames'}
              label={intl.formatMessage({ id: 'product.otherNames' })}
              rules={[{ required: false }]}
            >
              <Input disabled={disable} />
            </Form.Item>
            <Form.Item
              name={'productImage'}
              label={intl.formatMessage({ id: 'product.productImage' })}
              rules={[{ required: false }]}
            >
              <ImageUpload disabled={disable} onUpload={onUpload} />
            </Form.Item>
            <Form.Item
              name={'itemCategory'}
              label={intl.formatMessage({ id: 'product.itemCategory' })}
              rules={[{ required: true }]}
            >
              <Select disabled={disable}></Select>
            </Form.Item>
          </Form>
        </Tabs.TabPane>

        {/* label details */}
        <Tabs.TabPane tab={intl.formatMessage({ id: 'product.labelDetails' })} key="2">
          <Form
            initialValues={{ ...labelDetails }}
            ref={labelDetailsRef}
            labelAlign={'left'}
            wrapperCol={{ span: 16 }}
            labelCol={{ span: 8 }}
          >
            <Form.Item
              name={'upcCode'}
              label={intl.formatMessage({ id: 'product.upcCode' })}
              rules={[{ required: true }]}
            >
              <Input disabled={disable} />
            </Form.Item>
            <Form.Item
              name={'shelfLife'}
              label={intl.formatMessage({ id: 'product.shelfLife' })}
              rules={[{ required: true }]}
            >
              <Input disabled={disable} />
            </Form.Item>
            <Form.Item
              name={'shelfUOM'}
              label={intl.formatMessage({ id: 'product.shelfUOM' })}
              rules={[{ required: true }]}
            >
              <Select disabled={disable}>
                <Select.Option key={'Week'}>Week</Select.Option>
                <Select.Option key={'Month'}>Month</Select.Option>
              </Select>
            </Form.Item>
            <Form.Item
              name={'storageConditions'}
              label={intl.formatMessage({ id: 'product.storageConditions' })}
              rules={[{ required: true }]}
            >
              <Input disabled={disable} />
            </Form.Item>
          </Form>
        </Tabs.TabPane>

        {/* variants */}
        <Tabs.TabPane tab={intl.formatMessage({ id: 'product.variants' })} key="3">
          {!editProductData?.variants.length ? (
            <Typography.Text>
              {intl.formatMessage({ id: 'product.pleaseClickEditToAddVariants' })}
            </Typography.Text>
          ) : null}
          <Form
            initialValues={variants}
            ref={variantsRef}
            labelAlign={'left'}
            wrapperCol={{ span: 16 }}
            labelCol={{ span: 8 }}
          >
            {editProductData?.variants.map((e, index: number) => {
              return (
                <Collapse accordion expandIconPosition={'end'} key={index}>
                  <Panel header={e.variantName} key={index}>
                    <Form.Item
                      name={`${index}_variantAttribute`}
                      label={intl.formatMessage({ id: 'product.variantAttribute' })}
                    >
                      <Input disabled={disable} />
                    </Form.Item>
                    <Form.Item
                      name={`${index}_simpleSku`}
                      label={intl.formatMessage({ id: 'product.simpleSku' })}
                    >
                      <Input disabled={disable} />
                    </Form.Item>
                    <Form.Item
                      name={`${index}_weight`}
                      label={intl.formatMessage({ id: 'product.weight' })}
                    >
                      <Input
                        disabled={disable}
                        addonAfter={
                          <Select defaultValue={e.weightUom} disabled={disable}></Select>
                        }
                      />
                    </Form.Item>
                    <Row>
                      <Col span={8}>
                        <Form.Item
                          labelCol={{ span: 24 }}
                          label={intl.formatMessage({ id: 'product.packageDimensions' })}
                        />
                      </Col>
                      <Col span={16}>
                        <Form.Item
                          wrapperCol={{ span: 24 }}
                          name={`${index}_height`}
                          labelCol={{ span: 24 }}
                          label={intl.formatMessage({ id: 'product.height' })}
                        >
                          <Input
                            disabled={disable}
                            addonAfter={
                              <Select
                                disabled={disable}
                                defaultValue={e.heightUom}
                                className={styles.addOnSelect}
                              ></Select>
                            }
                          />
                        </Form.Item>
                        <Form.Item
                          name={`${index}_width`}
                          wrapperCol={{ span: 24 }}
                          labelCol={{ span: 24 }}
                          label={intl.formatMessage({ id: 'product.width' })}
                        >
                          <Input
                            disabled={disable}
                            addonAfter={
                              <Select
                                disabled={disable}
                                defaultValue={e.widthUom}
                                className={styles.addOnSelect}
                              ></Select>
                            }
                          />
                        </Form.Item>
                        <Form.Item
                          wrapperCol={{ span: 24 }}
                          labelCol={{ span: 24 }}
                          name={`${index}_length`}
                          label={intl.formatMessage({ id: 'product.length' })}
                        >
                          <Input
                            disabled={disable}
                            addonAfter={
                              <Select
                                disabled={disable}
                                defaultValue={e.lengthUom}
                                className={styles.addOnSelect}
                              ></Select>
                            }
                          />
                        </Form.Item>
                      </Col>
                    </Row>
                  </Panel>
                </Collapse>
              );
            })}
          </Form>
        </Tabs.TabPane>

        {/* pricing */}
        <Tabs.TabPane tab={intl.formatMessage({ id: 'product.pricing' })} key="4">
          {!editProductData?.variants.length ? (
            <Typography.Text>
              {intl.formatMessage({ id: 'product.pleaseClickEditToAddPricing' })}
            </Typography.Text>
          ) : null}
          <Form
            initialValues={pricing}
            ref={pricingRef}
            labelAlign={'left'}
            wrapperCol={{ span: 16 }}
            labelCol={{ span: 8 }}
          >
            {editProductData?.variants.map((e, index: number) => {
              return (
                <Collapse accordion expandIconPosition={'end'} key={index}>
                  <Panel header={e.variantName} key={index}>
                    <Form.Item
                      name={`${index}_unitPrice`}
                      label={intl.formatMessage({ id: 'product.unitPrice' })}
                    >
                      <Input disabled={disable} />
                    </Form.Item>
                    <Row justify={'space-between'}>
                      <Col span={8}>
                        <Form.Item
                          labelCol={{ span: 24 }}
                          label={intl.formatMessage({ id: 'product.tieredPricing' })}
                        />
                      </Col>
                      <Col span={16}>
                        {e.tierPricing?.map((tier, tierIndex: number) => {
                          return (
                            <Row
                              gutter={[16, 0]}
                              justify={'space-between'}
                              key={tierIndex}
                            >
                              <Col span={8}>
                                <Form.Item
                                  name={`${index}_${tierIndex}_qty`}
                                  wrapperCol={{ span: 24 }}
                                  labelCol={{ span: 24 }}
                                  label={intl.formatMessage({ id: 'product.qty' })}
                                >
                                  <Input disabled={disable} />
                                </Form.Item>
                              </Col>
                              <Col span={16}>
                                <Form.Item
                                  name={`${index}_${tierIndex}_price`}
                                  wrapperCol={{ span: 24 }}
                                  labelCol={{ span: 24 }}
                                  label={intl.formatMessage({ id: 'product.price' })}
                                >
                                  <Input disabled={disable} />
                                </Form.Item>
                              </Col>
                            </Row>
                          );
                        })}
                      </Col>
                    </Row>
                  </Panel>
                </Collapse>
              );
            })}
          </Form>
        </Tabs.TabPane>
      </Tabs>
    </Drawer>
  );
};

export default EditDrawer;
